-----------------------------------------------------------
--|> CROSSHAIR
-- : Places a crosshair on a nameplate.
-- : Filtered for player targets that are hostile player with support for icons, text etc attachments
-----------------------------------------------------------
--|> INIT
local _, NS = ...
local RC = LibStub("LibRangeCheck-2.0")
NS.Crosshair = {}
NS.Crosshair.Enabled = false

--|> Upvalue Globals
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
local DISABLED_FONT_COLOR = DISABLED_FONT_COLOR
local UIFrameFadeIn = UIFrameFadeIn
local IsItemInRange = IsItemInRange
local UnitIsUnit = UnitIsUnit
local C_NamePlate = C_NamePlate
local UnitClass = UnitClass
local UnitIsConnected = UnitIsConnected
local IsFlying = IsFlying
local IsUsableItem = IsUsableItem
local select = select

--> Hide
local function HideCrosshair()
  weizPVP_CrosshairFrame:Hide()
  weizPVP_CrosshairFrame.active = false
end

--> Show
local function ShowCrosshair()
  if NS.Options.Crosshair.Enabled == true then
    UIFrameFadeIn(weizPVP_CrosshairFrame, 0.1, 0, NS.Options.Crosshair.Alpha)
    weizPVP_CrosshairFrame:Show()
    weizPVP_CrosshairFrame.active = true
  else
    NS.Crosshair.Disable()
  end
end

--> CONFIGURE RANGE
local function ConfigureRange()
  if NS.Options.Crosshair.ShowRange then
    weizPVP_CrosshairFrame.RangeText:Show()
  else
    weizPVP_CrosshairFrame.RangeText:Hide()
  end
end

-->  SET CLASS COLORS
local targetClassColor = {}
local function SetCrosshairColors()
  if UnitIsConnected("target") == true then
    targetClassColor = RAID_CLASS_COLORS[select(2, UnitClass("target"))]
  else
    targetClassColor = DISABLED_FONT_COLOR
  end
  --: CORE
  weizPVP_CrosshairFrame.Core:SetVertexColor(targetClassColor.r, targetClassColor.g, targetClassColor.b, 0.3)
  weizPVP_CrosshairFrame.Core_ADD:SetVertexColor(targetClassColor.r, targetClassColor.g, targetClassColor.b, 0.8)
  --: LINES
  weizPVP_CrosshairFrame.TopLine:SetVertexColor(
    targetClassColor.r,
    targetClassColor.g,
    targetClassColor.b,
    NS.Options.Crosshair.LineAlpha
  )
  weizPVP_CrosshairFrame.BottomLine:SetVertexColor(
    targetClassColor.r,
    targetClassColor.g,
    targetClassColor.b,
    NS.Options.Crosshair.LineAlpha
  )
  weizPVP_CrosshairFrame.LeftLine:SetVertexColor(
    targetClassColor.r,
    targetClassColor.g,
    targetClassColor.b,
    NS.Options.Crosshair.LineAlpha
  )
  weizPVP_CrosshairFrame.RightLine:SetVertexColor(
    targetClassColor.r,
    targetClassColor.g,
    targetClassColor.b,
    NS.Options.Crosshair.LineAlpha
  )
  weizPVP_CrosshairFrame.TargetFX:SetVertexColor(targetClassColor.r, targetClassColor.g, targetClassColor.b, 0.8)
  NS.Crosshair.SetAlpha()
  --: NAME TEXT
  if NS.Options.Crosshair.NameEnabled then
    weizPVP_CrosshairFrame.NameText:SetTextColor(targetClassColor.r, targetClassColor.g, targetClassColor.b)
  end
end

-->  RANGE CHECK
local function getRangeInfo()
  local minRange, _ = RC:GetRange("target")
  local outputAlpha = 1
  if minRange then
    if minRange < 20 then
      outputAlpha = 0
    elseif minRange < 30 then
      outputAlpha = 0.5
    elseif minRange < 40 then
      outputAlpha = 0.8
    end
  end
  if minRange == 200 then
    minRange = "200+"
  elseif not minRange then
    minRange = "|cffffff00--|r"
  end
  return minRange, outputAlpha
end

--> NET
local function ShowNet5000()
  weizPVP_CrosshairFrame.NetOMatic.BG.Pulse:Play()
  weizPVP_CrosshairFrame.NetOMatic:SetAlpha(1)
end

local function HideNet5000()
  weizPVP_CrosshairFrame.NetOMatic.BG.Pulse:Stop()
  weizPVP_CrosshairFrame.NetOMatic:SetAlpha(0)
end

local function IsNetUsable()
  return (IsFlying("target") and IsItemInRange("Net-o-Matic 5000", "target") and IsUsableItem("Net-o-Matic 5000"))
end

--> Target Changed
local function TargetRangeCheck()
  --: Net-o-Matic 5000 Check
  if IsNetUsable() or NS.Options.DEBUG then
    ShowNet5000()
  else
    HideNet5000()
  end
  --: Range Check
  local rText, rAlpha = getRangeInfo()
  if weizPVP_CrosshairFrame.RangeText:GetText() ~= rText then
    weizPVP_CrosshairFrame.RangeText:SetText(rText)
  end
  if weizPVP_CrosshairFrame.RangeText:GetAlpha() ~= rAlpha then
    weizPVP_CrosshairFrame.RangeText:SetAlpha(rAlpha)
  end
end

--> Range Timer

local function RangeTimer()
  if weizPVP_CrosshairFrame.active ~= true then
    return
  end
  TargetRangeCheck()
end
local timer_ActiveTarget = C_Timer.NewTicker(0.5, RangeTimer)

--> Check KOS
local function CheckKOS()
  if NS.KosList[NS.GetUnitName("target")] then
    weizPVP_CrosshairFrame.FourArrowsKOS.FadeIn:Play()
    weizPVP_CrosshairFrame.FourArrowsKOS.Rotate:Play()
    weizPVP_CrosshairFrame.FourArrows:Hide()
    weizPVP_CrosshairFrame.FourArrowsKOS:Show()
    return true
  else
    weizPVP_CrosshairFrame.FourArrows.FadeIn:Play()
    weizPVP_CrosshairFrame.FourArrows.Rotate:Play()
    weizPVP_CrosshairFrame.FourArrows:Show()
    weizPVP_CrosshairFrame.FourArrowsKOS:Hide()
    return false
  end
end

--> Setup Crosshair On Nameplate
local function SetupCrosshairOnNameplate(nameplate)
  if not nameplate then
    return
  end
  --: Set on same level as nameplate
  weizPVP_CrosshairFrame.Core:SetWidth(nameplate:GetWidth() + 14)
  weizPVP_CrosshairFrame.Core:SetHeight(weizPVP_CrosshairFrame.Core:GetWidth())
  weizPVP_CrosshairFrame:ClearAllPoints()
  weizPVP_CrosshairFrame:SetParent(nameplate)
  weizPVP_CrosshairFrame:SetPoint("CENTER")
  --: Alpha
  if not weizPVP_CrosshairFrame:IsShown() then
    NS.Crosshair.SetAlpha()
    ShowCrosshair()
  end
  --: Animations
  weizPVP_CrosshairFrame:StopAnimating()
  weizPVP_CrosshairFrame.TargetFX:Show()
  weizPVP_CrosshairFrame.TargetFX.Splash:Play()
  --: KOS Check
  CheckKOS()
  SetCrosshairColors()
end

--> Refresh Name Text
local targetPlayerName
local function RefreshNameText()
  if NS.Options.Crosshair.NameEnabled then
    local displayName = NS.FormatPlayerNameOnly(targetPlayerName)
    weizPVP_CrosshairFrame.NameText:SetText(displayName)
    weizPVP_CrosshairFrame.NameText:Show()
  else
    weizPVP_CrosshairFrame.NameText:Hide()
  end
end

--> Refresh Guild Text
local function RefreshGuildText()
  if NS.Options.Crosshair.GuildEnabled then
    if select(1, GetGuildInfo("target")) then
      weizPVP_CrosshairFrame.GuildText:SetText("|cffcccccc" .. (select(1, GetGuildInfo("target")) or "") .. "|r")
      weizPVP_CrosshairFrame.GuildGradient:Show()
      weizPVP_CrosshairFrame.GuildTextBG:Show()
      weizPVP_CrosshairFrame.GuildText:Show()
    else
      weizPVP_CrosshairFrame.GuildGradient:Hide()
      weizPVP_CrosshairFrame.GuildTextBG:Hide()
      weizPVP_CrosshairFrame.GuildText:Hide()
    end
  else
    weizPVP_CrosshairFrame.GuildGradient:Hide()
    weizPVP_CrosshairFrame.GuildText:Hide()
    weizPVP_CrosshairFrame.GuildTextBG:Hide()
  end
end

--> VALID UNIT TARGETED
local function ValidUnitTargeted()
  if not (C_NamePlate.GetNamePlateForUnit("target") and true or nil) then
    HideCrosshair()
    return
  end
  targetPlayerName = NS.GetUnitName("target")
  TargetRangeCheck()
  RefreshNameText()
  RefreshGuildText()
  ShowCrosshair()
  SetupCrosshairOnNameplate(C_NamePlate.GetNamePlateForUnit("target"))
end

--|> EVENTS
-----------------------------------------------------------
-->  NAMEPLATE ADDED
function NS.Crosshair.NameplateAdded(unit)
  if UnitIsUnit("target", unit) then
    if not NS.IsUnitValidForTracking("target") then
      HideCrosshair()
      return
    end
    ValidUnitTargeted()
  end
end

--> NAMEPLATE REMOVED
function NS.Crosshair.NameplateRemoved(unit)
  if UnitIsUnit("target", unit) then
    HideCrosshair()
  end
end

--> NEW TARGET
function NS.Crosshair.NewTarget()
  if NS.IsUnitValidForTracking("target") then
    ValidUnitTargeted()
  else
    NS.Crosshair.Reset()
  end
end

--|> FUNCTIONS: SETTINGS + CONFIG
--> SET ALPHA
function NS.Crosshair.SetAlpha()
  weizPVP_CrosshairFrame:SetAlpha(NS.Options.Crosshair.Alpha)
  weizPVP_CrosshairFrame.TopLine:SetAlpha(NS.Options.Crosshair.LineAlpha)
  weizPVP_CrosshairFrame.BottomLine:SetAlpha(NS.Options.Crosshair.LineAlpha)
  weizPVP_CrosshairFrame.LeftLine:SetAlpha(NS.Options.Crosshair.LineAlpha)
  weizPVP_CrosshairFrame.RightLine:SetAlpha(NS.Options.Crosshair.LineAlpha)
end

--> SET LINE THICKNESS
function NS.Crosshair.SetLineThickness()
  weizPVP_CrosshairFrame.TopLine:SetIgnoreParentScale(true)
  weizPVP_CrosshairFrame.BottomLine:SetIgnoreParentScale(true)
  weizPVP_CrosshairFrame.LeftLine:SetIgnoreParentScale(true)
  weizPVP_CrosshairFrame.RightLine:SetIgnoreParentScale(true)
  weizPVP_CrosshairFrame.TopLine:SetWidth(NS.Options.Crosshair.LineThickness)
  weizPVP_CrosshairFrame.BottomLine:SetWidth(NS.Options.Crosshair.LineThickness)
  weizPVP_CrosshairFrame.LeftLine:SetHeight(NS.Options.Crosshair.LineThickness)
  weizPVP_CrosshairFrame.RightLine:SetHeight(NS.Options.Crosshair.LineThickness)
end

--> SET SCALE
function NS.Crosshair.SetScale()
  weizPVP_CrosshairFrame:SetScale(NS.Options.Crosshair.Scale)
end

--> RESET
function NS.Crosshair.Reset()
  HideCrosshair()
  weizPVP_CrosshairFrame:ClearAllPoints()
  targetPlayerName = nil
  targetClassColor = {}
end

--> ENABLE
function NS.Crosshair.Enable()
  if not NS.Options.Crosshair.Enabled then
    return
  end
  timer_ActiveTarget = C_Timer.NewTicker(0.5, RangeTimer)
  NS.Crosshair.Reset()
  NS.Crosshair.SetLineThickness()
  NS.Crosshair.SetScale()
  NS.Crosshair.SetAlpha()
  ConfigureRange()
  weizPVP_CrosshairFrame.NetOMatic:Show()
  NS.ResizeFrameToUiScale(weizPVP_CrosshairFrame)
  NS.Crosshair.Enabled = true
  NS.Crosshair.NewTarget()
end

--> DISABLE
function NS.Crosshair.Disable()
  NS.Crosshair.Reset()
  NS.Crosshair.Enabled = false
  NS.Crosshair.active = false
end
--> TOGGLE
function NS.Crosshair.Toggle()
  if NS.Crosshair.Enabled == true then
    NS.Crosshair.Reset()
    NS.Crosshair.Enabled = false
  else
    NS.Crosshair.Enable()
    NS.Crosshair.Enabled = true
  end
end

--> OnLoad
function NS.Crosshair.OnLoad()
  NS.Crosshair.Reset()
end
