-----------------------------------------------------------
--|> LISTS
--: Manages the lists of players shown
-----------------------------------------------------------
--|> Upvalue Globals
-----------------------------------------------------------
local _, NS = ...
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
local CopyTable = CopyTable
local InCombatLockdown, GetTime = InCombatLockdown, GetTime
local pairs, strlen, gsub, tinsert, sort, wipe = pairs, strlen, gsub, tinsert, sort, wipe
local string_gsub = string.gsub
local PixelUtil_SetStatusBarValue = PixelUtil.SetStatusBarValue
local SM = LibStub("LibSharedMedia-3.0")

--|> GLOBALS
-----------------------------------------------------------
NS.NearbyListSize = 0
NS.CurrentList = {}
NS.CurrentNameplates = {}
NS.NearbyList = {}
NS.ActiveList = {}
NS.InactiveList = {}
NS.ActiveDeadList = {}
NS.InactiveDeadList = {}
NS.PlayersOnBars = {}

--> Update Nearby Count
local NearbyCountTopColorLimit = 100
function NS.UpdateNearbyCount()
  NS.NearbyCount = NS.NearbyListSize
  if NS.NearbyCount < 100 then
    weizPVP_CoreBar.Title:SetText(
      "|cff" ..
        NS.GetColorValueFromGradient(
          (NS.NearbyCount / NearbyCountTopColorLimit),
          0.4,
          1,
          0,
          1,
          0.74,
          0,
          1,
          0.35,
          0,
          1,
          0,
          0.34,
          1,
          0,
          0
        ) ..
          NS.NearbyCount .. "|r"
    )
  else
    weizPVP_CoreBar.Title:SetText("|cffff0000" .. NS.NearbyCount .. "|r")
  end
  if NS.Options.Frames.AutoResize and NS.NearbyCount <= NS.Options.Bars.MaxNumBars then
    NS.AutoResize()
  end
end

--> Manage Bars Displayed
function NS.ManageBarsDisplayed()
  for i = 1, NS.Options.Bars.MaxNumBars do
    if i > NS.NearbyListSize then
      NS.CoreUI.Bar[i]:SetAlpha(0)
      NS.CoreUI.Bar[i]:SetValue(100)
      NS.CoreUI.Bar[i].DeadIcon:Hide()
      NS.CoreUI.Bar[i].DeadIcon:Hide()
      NS.CoreUI.Bar[i].RoleIcon:SetTexture("Interface/Addons/weizPVP/Media/Icons/unknown.tga", false)
      NS.CoreUI.Bar[i].Name:SetText("")
      NS.CoreUI.Bar[i].Button.Target = ""
      NS.CoreUI.Bar[i].Button:EnableMouse(false)
      NS.CoreUI.Bar[i].name = ""
      NS.CoreUI.Bar[i].displayName = ""
      if not InCombatLockdown() then
        NS.CoreUI.Bar[i].Button:SetAttribute("macrotext1", "")
      end
    end
  end
  NS.CoreUI.ChangeTargetIcon()
end

--> Manage List Timeouts
local ActiveListCount = 0
-- local expired = false
-- local removed = false
-- local expiredCount = 0
-- local iTmp = 0
-- local timestamp = GetTime()
function NS.ManageListTimeouts()
  local expired = false
  local removed = false
  local expiredCount = 0
  local count = 1
  local timestamp = GetTime()
  --: ACTIVE
  for player in pairs(NS.ActiveList) do
    if
      (timestamp - NS.ActiveList[player].TimeUpdated) > NS.Options.Sorting.NearbyActiveTimeout and
        NS.CurrentNameplates[player] == nil
     then
      NS.InactiveList[player] = NS.ActiveList[player]
      NS.InactiveList[player].TimeAdded = timestamp + (count * 0.001)
      NS.ActiveList[player] = nil
      count = count + 1
      expired = true
    end
  end
  ActiveListCount = count
  count = 0
  --: ACTIVE DEAD
  timestamp = GetTime()
  for player in pairs(NS.ActiveDeadList) do
    if (timestamp - NS.ActiveDeadList[player].TimeUpdated) > NS.Options.Sorting.NearbyActiveTimeout then
      NS.InactiveDeadList[player] = NS.ActiveDeadList[player]
      NS.InactiveDeadList[player].TimeAdded = timestamp + (count * 0.001)
      NS.ActiveDeadList[player] = nil
      expired = true
      count = count + 1
    end
  end
  --: INACTIVE
  timestamp = GetTime()
  for player in pairs(NS.InactiveList) do
    if (timestamp - NS.InactiveList[player].TimeUpdated) > NS.Options.Sorting.NearbyInactiveTimeout then
      NS.InactiveList[player] = nil
      NS.NearbyList[player] = nil
      NS.PlayerActiveCache[player] = nil
      expiredCount = expiredCount + 1
      expired = true
      removed = true
    end
  end
  --: INACTIVE DEAD
  timestamp = GetTime()
  for player in pairs(NS.InactiveDeadList) do
    if (timestamp - NS.InactiveDeadList[player].TimeUpdated) > NS.Options.Sorting.NearbyInactiveTimeout then
      NS.InactiveDeadList[player] = nil
      NS.NearbyList[player] = nil
      NS.PlayerActiveCache[player] = nil
      expiredCount = expiredCount + 1
      expired = true
      removed = true
    end
  end
  if expired or removed then
    NS.NearbyListSize = NS.NearbyListSize - expiredCount
    NS.SortNearbyList()
    NS.UpdateNearbyCount()
    NS.RefreshCurrentList()
    NS.CoreUI.ChangeTargetIcon()
  end
  if removed then
    NS.ManageBarsDisplayed()
  end
end

--> Format Level String
local function FormatLevelString(estimated, level)
  if not level then
    return
  end
  local levelText = NS.FormatRelativeLevelForDisplay(level)
  if estimated == true and level ~= 0 then
    levelText = levelText .. "|cFFFF00CC+|r"
  end
  return levelText
end

--> Update Bar
local darkenValue = 0.05
local guildTxtLength
local roleIcons = {
  ["TANK"] = "Interface/Addons/weizPVP/Media/Roles/tank.tga",
  ["DAMAGER"] = "Interface/Addons/weizPVP/Media/Roles/damager.tga",
  ["HEALER"] = "Interface/Addons/weizPVP/Media/Roles/healer.tga",
  ["UNKNOWN"] = "Interface/Addons/weizPVP/Media/Roles/unknown.tga"
}

-- local function UpdateBar(num, player, Alpha, Health, Class, Guild, Level, Estimated, Stealth, Dead, Role, GUID, Name)
local function UpdateBar(num, player, Alpha, Health, Class, Guild, Level, Estimated, _, Dead, Role, GUID, Name)
  if (not Name) or (not player) or (not GUID) then
    return
  end
  if NS.CoreUI.Bar[num] and GUID then
    --: SYNC INFO
    NS.CoreUI.Bar[num].displayName = string_gsub(Name, "-" .. NS.PlayerRealm, "")
    NS.CoreUI.Bar[num].name = Name
    NS.CoreUI.Bar[num].GUID = GUID
    NS.PlayersOnBars[GUID] = num
    --: TARGET MACRO
    if not InCombatLockdown() and NS.CoreUI.Bar[num].Button.Target ~= Name then
      NS.CoreUI.Bar[num].Button:SetAttribute("type1", "macro")
      NS.CoreUI.Bar[num].Button:SetAttribute("macrotext1", "/targetexact " .. NS.CoreUI.Bar[num].displayName)
      NS.CoreUI.Bar[num].Button.Target = Name
      NS.CoreUI.Bar[num].Button:EnableMouse(true)
    end
    --: ALPHA SET
    NS.CoreUI.Bar[num].Button:SetAlpha(0.2)
    if Alpha and Alpha ~= 0 then
      NS.CoreUI.Bar[num]:SetAlpha(Alpha)
    else
      NS.CoreUI.Bar[num]:SetAlpha(NS.Options.Bars.AlphaDefault)
    end
    --: KOS ICON
    if NS.KosList[Name] then
      NS.CoreUI.Bar[num].KOSRibbon:Show()
    else
      NS.CoreUI.Bar[num].KOSRibbon:Hide()
    end
    --: NAME TEXT
    local charName = gsub(NS.CoreUI.Bar[num].displayName, "-(.*)", "")
    -- if Stealth then
    -- charName = "|cFFffc863" .. charName .. "|r"
    -- elseif NS.PlayerActiveCache[GUID].Unattackable then
    --     charName = NS.ColorsLUT["unattackableLight"].sStart .. charName .. "|r"
    -- end
    -- if NS.PlayerActiveCache[GUID].Unattackable and NS.Options.Bars.ShowUnattackable == true then
    --   charName = "|cff" .. NS.Options.Bars.ShowUnattackableColor.hex .. charName .. "|r"
    -- end
    if strlen(NS.CoreUI.Bar[num].displayName) > strlen(charName) then
      NS.CoreUI.Bar[num].Name:SetText(charName .. "|cFFFF00CC*|r")
    else
      NS.CoreUI.Bar[num].Name:SetText(charName)
    end

    --: TAXI ICON
    if NS.PlayerActiveCache[GUID].OnTaxi then
      NS.CoreUI.Bar[num].Name:SetText(
        "|TInterface/Addons/weizPVP/Media/Icons/flight_path.tga:0|t " .. NS.CoreUI.Bar[num].Name:GetText()
      )
    end

    --: LEVEL
    NS.CoreUI.Bar[num].Level:SetText(FormatLevelString(Estimated, Level))

    --: CLASS COLOR (BAR COLOR)
    if Class and RAID_CLASS_COLORS[Class] then
      NS.CoreUI.Bar[num]:SetStatusBarColor(
        RAID_CLASS_COLORS[Class].r - darkenValue,
        RAID_CLASS_COLORS[Class].g - darkenValue,
        RAID_CLASS_COLORS[Class].b - darkenValue,
        Alpha
      )
    end
    NS.CoreUI.Bar[num].bg:SetVertexColor(0, 0, 0, 0.5)

    --: HEALTH (BAR VALUE)
    if Health ~= nil then
      PixelUtil_SetStatusBarValue(NS.CoreUI.Bar[num], Health)
    else
      PixelUtil_SetStatusBarValue(NS.CoreUI.Bar[num], 100)
    end

    --: ROLE
    if Role then
      NS.CoreUI.Bar[num].RoleIcon:SetTexture(roleIcons[Role])
    elseif NS.PlayerDB[Name].Role then
      NS.CoreUI.Bar[num].RoleIcon:SetTexture(roleIcons[NS.PlayerDB[Name].Role])
    else
      NS.CoreUI.Bar[num].RoleIcon:SetTexture(roleIcons["UNKNOWN"])
    end

    --: DEAD ICON
    if Dead ~= nil then
      if Dead == true then
        NS.CoreUI.Bar[num].DeadIcon:Show()
        NS.CoreUI.Bar[num]:SetValue(0)
      elseif Dead == false then
        NS.CoreUI.Bar[num].DeadIcon:Hide()
      end
    end
    --: GUILD TEXT
    guildTxtLength = NS.CoreUI.Bar[num].Level:GetWidth() + NS.CoreUI.Bar[num].Name:GetWidth() + 28
    if NS.CoreUI.Bar[num].DeadIcon:IsShown() then
      guildTxtLength = guildTxtLength + 22
    end
    guildTxtLength = NS.CoreUI.Bar[num]:GetWidth() - guildTxtLength
    NS.CoreUI.Bar[num].Guild:SetText(Guild)
    NS.CoreUI.Bar[num].Guild:SetWidth(guildTxtLength)
    NS.CoreUI.Bar[num]:SetAlpha(NS.Options.Bars.AlphaDefault)
  end
end

--> Update Player List
local function UpdatePlayerLists(GUID, timeUpdate, dead, newPlayerOnList)
  if not NS.Options.Bars then
    return
  end

  local reSortList = false
  local playerOnBar = true
  if NS.NearbyListSize <= NS.Options.Bars.MaxNumBars then
    if dead == true or newPlayerOnList == true then
      reSortList = true
    end
  else
    if NS.PlayersOnBars[GUID] ~= nil then
      if dead == true then
        reSortList = true
      end
    end
  end

  -- NEW PLAYER
  if NS.NearbyList[GUID] == nil or newPlayerOnList then -- ADDING NEW PLAYER
    playerOnBar = false
    reSortList = true
    --: New Player on KOS
    if NS.KosList[NS.PlayerActiveCache[GUID].Name] then
      -- Audio Alert
      if NS.Options.KOS.AudioAlert == true then
        NS.PlaySoundAlert(SM:Fetch("sound", NS.Options.KOS.AudioAlertFile), true)
      end
      -- Flash OS program icon (taskbar)
      if NS.Options.KOS.TaskbarAlert == true then
        FlashClientIcon()
      end
      -- Chat Alert
      if NS.Options.KOS.ChatAlert == true then
        NS.PrintAddonMessage(
          "|TInterface/Addons/weizPVP/Media/Icons/kos.tga::0|t " ..
            NS.FormatPlayerNameAndRealn(NS.PlayerActiveCache[GUID].Name) .. " |cff8fdaffdetected!|r "
        )
      end
    end
    --: New player
    if NS.Options.AudioAlerts.DetectedPlayerSound == true then -- NON-KOS PLAYER
      if not (NS.Options.AudioAlerts.DetectedPlayerSoundBGDisabled == true and NS.Zone.instance == "pvp") then
        if not (NS.Options.AudioAlerts.DisableInSanctuary == true and NS.Zone.pvpType == "sanctuary") then
          NS.PlaySoundAlert(SM:Fetch("sound", NS.Options.AudioAlerts.DetectedPlayerSoundFile))
        end
      end
    end
    NS.NearbyList[GUID] = {}
    NS.NearbyList[GUID].TimeUpdated = timeUpdate
    NS.NearbyList[GUID].TimeAdded = timeUpdate
    if dead == true then
      NS.ActiveDeadList[GUID] = NS.ActiveDeadList[GUID] or {}
      NS.ActiveDeadList[GUID].TimeUpdated = timeUpdate
      NS.ActiveDeadList[GUID].TimeAdded = timeUpdate
    else
      NS.ActiveList[GUID] = NS.ActiveList[GUID] or {}
      NS.ActiveList[GUID].TimeAdded = timeUpdate
      NS.ActiveList[GUID].TimeUpdated = timeUpdate
    end
  elseif not NS.ActiveList[GUID] and not NS.ActiveDeadList[GUID] then -- EXISTING PLAYER; WAS INACTIVE
    reSortList = true
    if dead then
      NS.ActiveDeadList[GUID] = NS.InactiveDeadList[GUID] or {}
      NS.ActiveDeadList[GUID].TimeUpdated = timeUpdate
      NS.ActiveDeadList[GUID].TimeAdded = timeUpdate
      NS.NearbyList[GUID].TimeUpdated = timeUpdate
      NS.NearbyList[GUID].TimeAdded = timeUpdate
      NS.ActiveList[GUID] = nil
    else
      NS.ActiveList[GUID] = NS.InactiveList[GUID] or {}
      NS.ActiveList[GUID].TimeUpdated = timeUpdate
      NS.ActiveList[GUID].TimeAdded = timeUpdate
      NS.NearbyList[GUID].TimeUpdated = timeUpdate
      NS.NearbyList[GUID].TimeAdded = timeUpdate
      NS.ActiveDeadList[GUID] = nil
    end
    NS.InactiveList[GUID] = nil
    NS.InactiveDeadList[GUID] = nil
  else -- EXISTING PLAYER; ACTIVE
    if dead then
      NS.ActiveDeadList[GUID] = NS.ActiveDeadList[GUID] or {}
      NS.ActiveDeadList[GUID].TimeUpdated = timeUpdate
      NS.NearbyList[GUID].TimeUpdated = timeUpdate
      NS.ActiveList[GUID] = nil
    else
      NS.ActiveList[GUID] = NS.ActiveList[GUID] or {}
      NS.ActiveList[GUID].TimeUpdated = timeUpdate
      NS.NearbyList[GUID].TimeUpdated = timeUpdate
      NS.ActiveDeadList[GUID] = nil
    end
  end
  --> Check to see if we need to add a kos player to the bars
  if playerOnBar == false and ActiveListCount > NS.Options.Bars.MaxNumBars then
    if not NS.KosList[NS.PlayerActiveCache[GUID].Name] then
      return
    else
      reSortList = true
      newPlayerOnList = true
    end
  end
  -- Sort only if we moved the player from one sub-list to another
  if reSortList == true then
    NS.SortNearbyList()
    if NS.PlayersOnBars[GUID] or newPlayerOnList then
      NS.RefreshCurrentList()
    end
  else
    if NS.PlayersOnBars[GUID] then
      NS.RefreshBarByGUID(GUID)
    end
  end
end

--> Add Player Data
local lastTimestamp = GetTime()
function NS.AddPlayerDataToNearby(GUID, newPlayerOnList)
  if not GUID then
    return
  end
  local dead = false
  if NS.PlayerActiveCache[GUID] and NS.PlayerActiveCache[GUID].Dead ~= nil then
    dead = NS.PlayerActiveCache[GUID].Dead
  else
    NS.PlayerActiveCache[GUID].Dead = false
  end
  local time = GetTime()
  if lastTimestamp >= time then
    lastTimestamp = lastTimestamp + 0.0001
  else
    lastTimestamp = time
  end
  if newPlayerOnList then
    NS.NearbyListSize = NS.NearbyListSize + 1
    NS.UpdateNearbyCount()
  end
  UpdatePlayerLists(GUID, lastTimestamp, dead, newPlayerOnList)
end

--> Refresh Current List
function NS.RefreshCurrentList()
  -- Refreshing all bars due to required sorting
  local localPlayersOnBars = {}
  for k, data in pairs(NS.CurrentList) do
    if k <= NS.Options.Bars.MaxNumBars then
      -- Update Alpha if needed
      local alpha = NS.Options.Bars.AlphaDefault or 1
      if NS.InactiveList[data.player] or NS.InactiveDeadList[data.player] then
        alpha = NS.Options.Bars.AlphaInactive
      elseif NS.ActiveDeadList[data.player] then
        alpha = NS.Options.Bars.AlphaDead
      end
      if NS.PlayerActiveCache[data.player] then
        localPlayersOnBars[NS.PlayerActiveCache[data.player].GUID] = k
        local playerName = NS.PlayerActiveCache[data.player].Name or nil
        -- Update Bar
        if NS.PlayerDB[playerName] then
          UpdateBar(
            k,
            data.player,
            alpha,
            NS.PlayerActiveCache[data.player].Health,
            NS.PlayerDB[playerName].Class,
            NS.PlayerDB[playerName].Guild,
            NS.PlayerDB[playerName].Level,
            NS.PlayerDB[playerName].Estimated,
            NS.PlayerActiveCache[data.player].Stealth,
            NS.PlayerActiveCache[data.player].Dead,
            NS.PlayerDB[playerName].Role,
            data.player,
            playerName
          )
        end
      end
    else
      break
    end
  end
  NS.PlayersOnBars = CopyTable(localPlayersOnBars)
end

--> Sort Nearby List

function NS.SortNearbyList()
  local tempCurrentList = {}
  local tempActiveList = {}
  local tempActiveDeadList = {}
  local tempInactiveList = {}
  local tempInactiveDeadList = {}
  local tempPosition

  --: ActiveList
  for player in pairs(NS.ActiveList) do
    if NS.NearbyList[player] then
      tempPosition = NS.NearbyList[player].TimeAdded or nil
      if tempPosition ~= nil then
        tinsert(tempActiveList, {player = player, time = tempPosition})
      end
    end
  end
  --: ActiveDeadList
  for player in pairs(NS.ActiveDeadList) do
    if NS.NearbyList[player] then
      tempPosition = NS.NearbyList[player].TimeAdded or nil
      if tempPosition ~= nil then
        tinsert(tempActiveDeadList, {player = player, time = tempPosition})
      end
    end
  end
  --: InactiveDeadList
  for player in pairs(NS.InactiveDeadList) do
    if NS.NearbyList[player] then
      tempPosition = NS.NearbyList[player].TimeAdded or nil
      if tempPosition ~= nil then
        tinsert(tempInactiveDeadList, {player = player, time = tempPosition})
      end
    end
  end
  --: InactiveList
  for player in pairs(NS.InactiveList) do
    if NS.NearbyList[player] then
      tempPosition = NS.NearbyList[player].TimeAdded
      if tempPosition ~= nil then
        tinsert(tempInactiveList, {player = player, time = tempPosition})
      end
    end
  end
  --: sorts
  sort(
    tempActiveList,
    function(a, b)
      return a.time < b.time
    end
  )
  sort(
    tempActiveDeadList,
    function(a, b)
      return a.time < b.time
    end
  )
  sort(
    tempInactiveList,
    function(a, b)
      return a.time < b.time
    end
  )
  sort(
    tempInactiveDeadList,
    function(a, b)
      return a.time < b.time
    end
  )
  --: create player list
  for player in pairs(tempActiveList) do
    if NS.PlayerActiveCache[tempActiveList[player].player] then
      if NS.KosList[NS.PlayerActiveCache[tempActiveList[player].player].Name] then
        tinsert(tempCurrentList, tempActiveList[player])
        tempActiveList[player] = nil
      end
    end
  end
  for player in pairs(tempActiveList) do
    tinsert(tempCurrentList, tempActiveList[player])
  end
  for player in pairs(tempActiveDeadList) do
    tinsert(tempCurrentList, tempActiveDeadList[player])
  end
  for player in pairs(tempInactiveList) do
    tinsert(tempCurrentList, tempInactiveList[player])
  end
  for player in pairs(tempInactiveDeadList) do
    tinsert(tempCurrentList, tempInactiveDeadList[player])
  end
  NS.CurrentList = CopyTable(tempCurrentList)
  -- wipe(tempCurrentList)
  -- wipe(tempActiveList)
  -- wipe(tempActiveDeadList)
  -- wipe(tempInactiveList)
  -- wipe(tempInactiveDeadList)
end

--> Clear List Data
function NS.ClearListData()
  if NS.NearbyCount ~= 0 then
    wipe(NS.CurrentList)
    wipe(NS.CurrentNameplates)
    wipe(NS.NearbyList)
    wipe(NS.ActiveList)
    wipe(NS.InactiveList)
    wipe(NS.ActiveDeadList)
    wipe(NS.InactiveDeadList)
    wipe(NS.PlayersOnBars)
    wipe(NS.PlayerActiveCache)
    NS.NearbyListSize = 0
    NS.SortNearbyList()
    NS.ManageBarsDisplayed()
    NS.RefreshCurrentList()
    NS.UpdateNearbyCount()
    weizPVP_CoreFrame.ScrollFrame:SetVerticalScroll(0)
    NS:PlayerTargetEvent()
    if NS.Options.Frames.AutoResize == true then
      NS.AutoResize()
    end
    NS.CoreUI.ChangeTargetIcon()
  end
end

--> Refresh One Bar
function NS.RefreshBarByGUID(GUID)
  if not GUID then
    return
  end
  if not NS.PlayersOnBars[GUID] then
    return
  end
  --: Alpha
  local Alpha = 1
  if not NS.Options.Bars then
    Alpha = 1
  end
  if NS.Options.Bars then
    Alpha = NS.Options.Bars.AlphaDefault
  end
  if NS.InactiveList[GUID] or NS.InactiveDeadList[GUID] then
    Alpha = NS.Options.Bars.AlphaInactive or 0.6
  end
  if NS.ActiveDeadList[GUID] then
    Alpha = NS.Options.Bars.AlphaDead or 0.8
  end
  --: Update Bar
  local name = NS.PlayerActiveCache[GUID].Name
  UpdateBar(
    NS.PlayersOnBars[GUID],
    GUID,
    Alpha,
    NS.PlayerActiveCache[GUID].Health,
    NS.PlayerDB[name].Class,
    NS.PlayerDB[name].Guild,
    NS.PlayerDB[name].Level,
    NS.PlayerDB[name].Estimated,
    NS.PlayerActiveCache[GUID].Stealth,
    NS.PlayerActiveCache[GUID].Dead,
    NS.PlayerDB[name].Role,
    GUID,
    name
  )
end
