-----------------------------------------------------------
--|> DATA PROCESSING
--: Functions that help acquire and manage data involving the PlayerDB
-----------------------------------------------------------
local _, NS = ... --> Addon Name, Namespace

--> UpdatePlayerActiveCache
--: Updates data in the PlayerActiveCache and then calls
--: Sends data off the Lists to be processed

function NS.UpdatePlayerActiveCache(name, stealth, dead, role, GUID)
  --> Verify GUID exists
  if not GUID then
    return
  end
  local newPlayerOnList = true
  --: Check for player already in cache
  if NS.PlayerActiveCache[GUID] then
    newPlayerOnList = false
  else
    NS.PlayerActiveCache[GUID] = {}
  end
  --: Update PlayerCache info
  if newPlayerOnList then
    NS.PlayerActiveCache[GUID].GUID = GUID
    --: NAME
    if name then
      NS.PlayerActiveCache[GUID].Name = name
    elseif NS.PlayerDB[name].Name then
      NS.PlayerActiveCache[GUID].Name = NS.PlayerDB[name].Name
    end
    --: LEVEL
    if NS.PlayerDB[name].Level then
      NS.PlayerActiveCache[GUID].Level = NS.PlayerDB[name].Level
      NS.PlayerActiveCache[GUID].Estimated = NS.PlayerDB[name].Estimated
    else -- no level found
      NS.PlayerActiveCache[GUID].Level = 0
      NS.PlayerActiveCache[GUID].Estimated = true
      NS.PlayerDB[name].Level = 0
      NS.PlayerDB[name].Estimated = true
    end
    --: CLASS
    if NS.PlayerDB[name].Class then
      NS.PlayerActiveCache[GUID].Class = NS.PlayerDB[name].Class
    end
    --: GUILD
    if NS.PlayerDB[name].Guild then
      NS.PlayerActiveCache[GUID].Guild = NS.PlayerDB[name].Guild
    end
    --: STEALTH
    if stealth ~= nil then
      NS.PlayerActiveCache[GUID].Stealth = stealth
    end
    --: DEAD
    if dead ~= nil then
      NS.PlayerActiveCache[GUID].Dead = dead
      if dead == true then
        NS.PlayerActiveCache[GUID].Health = 0
      elseif dead == false and NS.PlayerActiveCache[GUID].Health == 0 then
        NS.PlayerActiveCache[GUID].Health = 1
      end
    end
    --: ROLE
    if role ~= nil then
      NS.PlayerActiveCache[GUID].Role = role
      NS.PlayerDB[name].Role = role
    end
  end
  NS.AddPlayerDataToNearby(GUID, newPlayerOnList)
end

--> Static Role Assignment
function NS.ClassRoleAssign(class)
  if class == "ROGUE" or class == "MAGE" or class == "WARLOCK" or class == "HUNTER" then
    return "DAMAGER"
  end
  return nil
end
