-----------------------------------------------------------
--|> UNIT PROCESSING
--: Process Unit Data
-----------------------------------------------------------
local _, NS = ... --> Addon Name, Namespace
--> UpValues
local UnitName = UnitName
local GetGuildInfo = GetGuildInfo
local GetTime = GetTime
local UnitCanAttack = UnitCanAttack
local UnitExists = UnitExists
local UnitClass = UnitClass
local UnitRace = UnitRace
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitOnTaxi = UnitOnTaxi
local UnitIsPlayer = UnitIsPlayer
local GetUnitName = GetUnitName
local UnitLevel = UnitLevel
local gsub = gsub
local select = select

--> GET UNIT DATA
local unitUpdateThreshold = 120 -- 2 minutes between guild/race/level/role checks
local gudName
local gudGUID
function NS.GetUnitData(unit)
  if (not unit) or (not UnitExists(unit)) or (not NS.IsUnitValidForTracking(unit)) then
    return -- no valid unit to process
  end
  gudName = NS.GetUnitName(unit) or nil
  gudGUID = UnitGUID(unit) or nil
  if gudName and gudGUID then
    --: Add player to DB if not found
    if not NS.PlayerDB[gudName] then
      NS.PlayerDB[gudName] = {}
      NS.PlayerDB[gudName].Name = gudName
      NS.PlayerDB[gudName].Timestamp = 0
      _, NS.PlayerDB[gudName].Class = UnitClass(unit)
      NS.PlayerDB[gudName].Role = NS.ClassRoleAssign(NS.PlayerDB[gudName].Class)
    end

    --: Update player info if estimated or past update threshold
    if NS.PlayerDB[gudName].Timestamp + unitUpdateThreshold > GetTime() or NS.PlayerDB[gudName].Estimated == true then
      NS.PlayerDB[gudName].Guild = GetGuildInfo(unit)
      NS.PlayerDB[gudName].Level = UnitLevel(unit)
      NS.PlayerDB[gudName].Race = UnitRace(unit)
    end
    --: Player On Bars?
    if NS.PlayersOnBars[gudGUID] then
      NS.PlayerActiveCache[gudGUID].OnTaxi = UnitOnTaxi(unit) or nil
      NS.UnitHealthCheck(unit)
    end
    NS.PlayerDB[gudName].Estimated = false
    NS.PlayerDB[gudName].Timestamp = GetTime()
    NS.UpdatePlayerActiveCache(gudName, nil, nil, nil, gudGUID) -- (name, stealth, dead, role, GUID)
  end
  gudName, gudGUID = nil, nil
end

--> Is Unit Valid For Tracking
local iuvGUID
function NS.IsUnitValidForTracking(unit)
  if (not unit) or (not UnitIsPlayer(unit)) then -- input check
    return false
  end
  if GetUnitName(unit) == "Unknown" then
    return false
  end
  if (UnitCanAttack("player", unit) or UnitIsEnemy("player", unit) or NS.Options.DEBUG) then -- enemy player check
    return true
  else
    if not NS.Options.DEBUG and NS.PlayerActiveCache[UnitGUID(unit)] then
      iuvGUID = UnitGUID(unit) or nil
      NS.PlayerActiveCache[iuvGUID] = nil
      if NS.ActiveList[iuvGUID] then
        NS.ActiveList[iuvGUID].TimeAdded = 0
        NS.ActiveList[iuvGUID].TimeUpdated = 0
      end
      if NS.ActiveDeadList[iuvGUID] then
        NS.ActiveDeadList[iuvGUID].TimeAdded = 0
        NS.ActiveDeadList[iuvGUID].TimeUpdated = 0
      end
      if NS.InactiveList[iuvGUID] then
        NS.InactiveList[iuvGUID].TimeAdded = 0
        NS.InactiveList[iuvGUID].TimeUpdated = 0
      end
      if NS.InactiveDeadList[iuvGUID] then
        NS.InactiveDeadList[iuvGUID].TimeAdded = 0
        NS.InactiveDeadList[iuvGUID].TimeUpdated = 0
      end
      NS.CurrentNameplates[iuvGUID] = nil
      if NS.NearbyCount == 1 then
        NS.CoreUI.Initialize()
      end
      NS.ManageListTimeouts()
    end
  end
  return false
end

--> weizPVP: GetUnitName()
--: @returns "playerName-realmName" for all players, no matter the realm
local name, realm
function NS.GetUnitName(unit)
  if not unit then -- check for unit
    return
  end
  name, realm = UnitName(unit, true)
  if not name then
    return
  end
  if not realm then --: same realm
    return name .. "-" .. NS.PlayerRealm
  else --: different realm
    realm = gsub(realm, "[%s%-]", "")
    return name .. "-" .. realm
  end
end

--> UnitHealthCheck
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local UnitGUID = UnitGUID
local GUID
function NS.UnitHealthCheck(unit)
  GUID = UnitGUID(unit) or nil
  if GUID and UnitExists(unit) then
    NS.PlayerActiveCache[GUID].Dead = UnitIsDeadOrGhost(unit)
    NS.PlayerActiveCache[GUID].Health = UnitHealth(unit) / UnitHealthMax(unit) * 100
    NS.RefreshBarByGUID(GUID)
  end
end

--> Unescape
local escapes = {
  ["|c%x%x%x%x%x%x%x%x"] = "", -- color start
  ["|r"] = "", -- color end
  ["|H.-|h(.-)|h"] = "%1", -- links
  ["|T.-|t"] = "", -- textures
  ["{.-}"] = "" -- raid target icons
}

function NS.Unescape(str)
  for k, v in pairs(escapes) do
    str = gsub(str, k, v)
  end
  return str
end

--> GET COLOR VALUE FROM GRADIENT
--: value = 0-1 value
--: ... = color values
local rGrad, gGrad, bGrad
local numGrad
local segment, relperc
local r1, g1, b1, r2, g2, b2 = 1, 0, 0, 1, 0, 0
function NS.GetColorValueFromGradient(value, ...)
  if value >= 1 then
    rGrad, gGrad, bGrad = select(select("#", ...) - 2, ...)
    NS.RGB2Hex(abs(rGrad * 255), abs(gGrad * 255), abs(bGrad * 255))
  elseif value <= 0 then
    rGrad, gGrad, bGrad = ...
    NS.RGB2Hex(abs(rGrad * 255), abs(gGrad * 255), abs(bGrad * 255))
  end
  numGrad = select("#", ...) / 3
  segment, relperc = math.modf(value * (numGrad - 1))
  r1, g1, b1, r2, g2, b2 = select((segment * 3) + 1, ...)
  rGrad = r1 + (r2 - r1) * relperc
  gGrad = g1 + (g2 - g1) * relperc
  bGrad = b1 + (b2 - b1) * relperc
  return NS.RGB2Hex(abs(rGrad * 255), abs(gGrad * 255), abs(bGrad * 255))
end

--> RGB TO HSV
function NS.RGB2Hex(r, g, b)
  r = r <= 255 and r >= 0 and r or 0
  g = g <= 255 and g >= 0 and g or 0
  b = b <= 255 and b >= 0 and b or 0
  return string.format("%02x%02x%02x", r, g, b)
end

--> RESCALE FRAME
local function RescaleRegionSize(region)
  if not region then
    return
  end
  if not region.rescaled then
    region:SetWidth(NS.ScalePixelsToUi(region:GetWidth()))
    region:SetHeight(NS.ScalePixelsToUi(region:GetHeight()))
    region.rescaled = true
  end
end

function NS.ResizeFrameToUiScale(frame)
  if (not frame) or (not frame.rescaled) then
    return
  end
  RescaleRegionSize(frame)
  return
end

--> GET UI MULTIPLIER
---------------------------------------
function NS.GetUiMultiplier()
  return ConvertPixelsToUI(1, UIParent:GetEffectiveScale())
end

--> SCALE PIXELS TO WIDTH
---------------------------------------
function NS.ScalePixelsToUi(pixels)
  if not pixels then
    return nil
  end
  return (NS.GetUiMultiplier() * pixels)
end
